@@ -17,15 +17,6 @@ pub enum InputRequest {
     DeletePrevWord,
     DeleteNextWord,
     DeleteLine,
-    Submit,
-    Escape,
-}
-
-#[derive(Debug, PartialOrd, PartialEq, Eq, Clone, Copy, Hash)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct StateChanged {
-    pub value: bool,
-    pub cursor: bool,
 }
 
 /// Input response is emitted to notify about state changes and other events.
@@ -33,13 +24,11 @@ pub struct StateChanged {
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub enum InputResponse {
     /// The value or cursor or both changed.
-    StateChanged(StateChanged),
-
-    /// Enter was pressed.
-    Submitted,
-
-    /// Esc was pressed.
-    Escaped,
+    StateChanged {
+        value: bool,
+        cursor: bool,
+    },
+    Unchanged,
 }
 
 /// The input buffer with cursor support.
@@ -79,19 +68,19 @@ impl Input {
     }
 
     /// Handle request and emit response.
-    pub fn handle(&mut self, req: InputRequest) -> Option<InputResponse> {
+    pub fn handle(&mut self, req: InputRequest) -> InputResponse {
         use InputRequest::*;
         match req {
             SetCursor(pos) => {
                 let pos = pos.min(self.value.chars().count());
                 if self.cursor == pos {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor = pos;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
             InsertChar(c) => {
@@ -109,15 +98,15 @@ impl Input {
                         .collect();
                 }
                 self.cursor += 1;
-                Some(InputResponse::StateChanged(StateChanged {
+                InputResponse::StateChanged {
                     value: true,
                     cursor: true,
-                }))
+                }
             }
 
             DeletePrevChar => {
                 if self.cursor == 0 {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor -= 1;
                     self.value = self
@@ -128,16 +117,16 @@ impl Input {
                         .map(|(_, c)| c)
                         .collect();
 
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: true,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             DeleteNextChar => {
                 if self.cursor == self.value.chars().count() {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.value = self
                         .value
@@ -146,28 +135,28 @@ impl Input {
                         .filter(|(i, _)| i != &self.cursor)
                         .map(|(_, c)| c)
                         .collect();
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: true,
                         cursor: false,
-                    }))
+                    }
                 }
             }
 
             GoToPrevChar => {
                 if self.cursor == 0 {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor -= 1;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             GoToPrevWord => {
                 if self.cursor == 0 {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor = self
                         .value
@@ -180,28 +169,28 @@ impl Input {
                         .skip_while(|c| !c.is_alphanumeric())
                         .skip_while(|c| c.is_alphanumeric())
                         .count();
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             GoToNextChar => {
                 if self.cursor == self.value.chars().count() {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor += 1;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             GoToNextWord => {
                 if self.cursor == self.value.chars().count() {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor = self
                         .value
@@ -212,30 +201,31 @@ impl Input {
                         .find(|(_, c)| c.is_alphanumeric())
                         .map(|(i, _)| i)
                         .unwrap_or_else(|| self.value.chars().count());
-                    Some(InputResponse::StateChanged(StateChanged {
+
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             DeleteLine => {
                 if self.value.is_empty() {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     let cursor = self.cursor;
                     self.value = "".into();
                     self.cursor = 0;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: true,
                         cursor: self.cursor == cursor,
-                    }))
+                    }
                 }
             }
 
             DeletePrevWord => {
                 if self.cursor == 0 {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     let remaining = self.value.chars().skip(self.cursor);
                     let rev = self
@@ -253,16 +243,16 @@ impl Input {
                     self.value =
                         rev.into_iter().rev().chain(remaining).collect();
                     self.cursor = rev_len;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: true,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             DeleteNextWord => {
                 if self.cursor == self.value.chars().count() {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.value = self
                         .value
@@ -277,41 +267,37 @@ impl Input {
                         )
                         .collect();
 
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: true,
                         cursor: false,
-                    }))
+                    }
                 }
             }
 
             GoToStart => {
                 if self.cursor == 0 {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor = 0;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
 
             GoToEnd => {
                 let count = self.value.chars().count();
                 if self.cursor == count {
-                    None
+                    InputResponse::Unchanged
                 } else {
                     self.cursor = count;
-                    Some(InputResponse::StateChanged(StateChanged {
+                    InputResponse::StateChanged {
                         value: false,
                         cursor: true,
-                    }))
+                    }
                 }
             }
-
-            Submit => Some(InputResponse::Submitted),
-
-            Escape => Some(InputResponse::Escaped),
         }
     }
 
@@ -331,6 +317,7 @@ mod tests {
 
     const TEXT: &str = "first second, third.";
 
+    use super::InputResponse::*;
     use super::*;
 
     #[test]
@@ -342,10 +329,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: false,
-                cursor: true
-            }))
+                cursor: true,
+            }
         );
 
         assert_eq!(input.value(), "first second, third.");
@@ -357,17 +344,17 @@ mod tests {
         assert_eq!(input.cursor(), TEXT.chars().count());
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: false,
-                cursor: true
-            }))
+                cursor: true,
+            }
         );
 
         let req = InputRequest::SetCursor(TEXT.chars().count());
         let resp = input.handle(req);
 
         assert_eq!(input.cursor(), TEXT.chars().count());
-        assert_eq!(resp, None);
+        assert_eq!(resp, Unchanged);
     }
 
     #[test]
@@ -379,10 +366,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: true,
-                cursor: true
-            }))
+                cursor: true,
+            }
         );
 
         assert_eq!(input.value(), "first second, third.x");
@@ -410,10 +397,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: false,
-                cursor: true
-            }))
+                cursor: true,
+            }
         );
 
         assert_eq!(input.value(), "first second, third.");
@@ -438,10 +425,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: true,
                 cursor: true,
-            }))
+            }
         );
 
         assert_eq!(input.value(), "¡test");
@@ -454,10 +441,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: true,
                 cursor: false,
-            }))
+            }
         );
 
         assert_eq!(input.value(), "test");
@@ -474,10 +461,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: true,
                 cursor: true,
-            }))
+            }
         );
 
         assert_eq!(input.value(), "¡test☆¡");
@@ -491,10 +478,10 @@ mod tests {
 
         assert_eq!(
             resp,
-            Some(InputResponse::StateChanged(StateChanged {
+            StateChanged {
                 value: true,
                 cursor: true,
-            }))
+            }
         );
 
         assert_eq!(input.value(), "¡☆test☆¡");
